@@ -1,6 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2000, 2001, 2002, 2005, 2006, 2007, 2008, 2009,
-   2010 Free Software Foundation, Inc.
+   Copyright (C) 1999-2002, 2005-2012, 2014-2017 Free Software
+   Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -13,9 +13,8 @@
    Lesser General Public License for more details.
 
    You should have received a copy of the GNU Lesser General
-   Public License along with this library; if not, write to the
-   Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110-1301 USA */
+   Public License along with this library.  If not, see
+   <http://www.gnu.org/licenses/>. */
 
 #ifdef HAVE_CONFIG_H
 # include <config.h>
@@ -48,7 +47,7 @@ sieve_mark_deleted (mu_message_t msg, int deleted)
 
 
 static int
-sieve_action_stop (mu_sieve_machine_t mach, mu_list_t args, mu_list_t tags)
+sieve_action_stop (mu_sieve_machine_t mach)
 {
   mu_sieve_log_action (mach, "STOP", NULL);
   mach->pc = 0;
@@ -56,7 +55,7 @@ sieve_action_stop (mu_sieve_machine_t mach, mu_list_t args, mu_list_t tags)
 }
 
 static int
-sieve_action_keep (mu_sieve_machine_t mach, mu_list_t args, mu_list_t tags)
+sieve_action_keep (mu_sieve_machine_t mach)
 {
   mu_sieve_log_action (mach, "KEEP", NULL);
   if (mu_sieve_is_dry_run (mach))
@@ -66,7 +65,7 @@ sieve_action_keep (mu_sieve_machine_t mach, mu_list_t args, mu_list_t tags)
 }
 
 static int
-sieve_action_discard (mu_sieve_machine_t mach, mu_list_t args, mu_list_t tags)
+sieve_action_discard (mu_sieve_machine_t mach)
 {
   mu_sieve_log_action (mach, "DISCARD", _("marking as deleted"));
   if (mu_sieve_is_dry_run (mach))
@@ -76,37 +75,33 @@ sieve_action_discard (mu_sieve_machine_t mach, mu_list_t args, mu_list_t tags)
 }
 
 static int
-sieve_action_fileinto (mu_sieve_machine_t mach, mu_list_t args, mu_list_t tags)
+sieve_action_fileinto (mu_sieve_machine_t mach)
 {
   int rc;
   int mbflags = 0;
-  mu_sieve_value_t *opt;
-  mu_sieve_value_t *val = mu_sieve_value_get (args, 0);
-  if (!val)
-    {
-      mu_sieve_error (mach, _("cannot get filename!"));
-      mu_sieve_abort (mach);
-    }
+  char *filename;
+  char *perms;
 
-  if (mu_sieve_tag_lookup (tags, "permissions", &opt))
+  mu_sieve_get_arg (mach, 0, SVT_STRING, &filename);
+
+  if (mu_sieve_get_tag (mach, "permissions", SVT_STRING, &perms))
     {
       const char *p;
       
-      if (mu_parse_stream_perm_string (&mbflags, opt->v.string, &p))
+      if (mu_parse_stream_perm_string (&mbflags, perms, &p))
 	{
 	  /* Should not happen, but anyway... */
 	  mu_sieve_error (mach, _("invalid permissions (near %s)"), p);
-	  return 1;
+	  mu_sieve_abort (mach);
 	}
     }
   
   mu_sieve_log_action (mach, "FILEINTO",
-		       _("delivering into %s"), val->v.string);
+		       _("delivering into %s"), filename);
   if (mu_sieve_is_dry_run (mach))
     return 0;
 
-  rc = mu_message_save_to_mailbox (mach->msg, mach->debug,
-				   val->v.string, mbflags);
+  rc = mu_message_save_to_mailbox (mach->msg, filename, mbflags);
   if (rc)
     mu_sieve_error (mach, _("cannot save to mailbox: %s"),
 		    mu_strerror (rc));
@@ -145,34 +140,35 @@ mime_create_reason (mu_mime_t mime, mu_message_t msg, const char *text)
   time_t t;
   struct tm *tm;
   char *sender;
-  mu_off_t off = 0;
   mu_body_t body;
   mu_header_t hdr;
-  char datestr[80];
   static char *content_header =
     "Content-Type: text/plain;charset=" MU_SIEVE_CHARSET "\n"
     "Content-Transfer-Encoding: 8bit\n";
  
   mu_message_create (&newmsg, NULL);
   mu_message_get_body (newmsg, &body);
-  mu_body_get_stream (body, &stream);
+  mu_body_get_streamref (body, &stream);
 
   time (&t);
   tm = localtime (&t);
-  mu_strftime (datestr, sizeof datestr, "%a, %b %d %H:%M:%S %Y %Z", tm);
 
   mu_sieve_get_message_sender (msg, &sender);
 
-  mu_stream_printf (stream, &off,
-		    "The original message was received at %s from %s.\n",
-		    datestr, sender);
+  mu_c_streamftime (stream,
+		    "The original message was received at "
+		    "%a, %b %d %H:%M:%S %Y %Z", tm, NULL);
+  
+  mu_stream_printf (stream, " from %s.\n", sender);
   free (sender);
-  mu_stream_printf (stream, &off,
+  mu_stream_printf (stream,
 		    "Message was refused by recipient's mail filtering program.\n");
-  mu_stream_printf (stream, &off, "Reason given was as follows:\n\n");
-  mu_stream_printf (stream, &off, "%s", text);
+  mu_stream_printf (stream, "Reason given was as follows:\n\n");
+  mu_stream_printf (stream, "%s", text);
   mu_stream_close (stream);
-  mu_header_create (&hdr, content_header, strlen (content_header), newmsg);
+  mu_stream_destroy (&stream);
+  
+  mu_header_create (&hdr, content_header, strlen (content_header));
   mu_message_set_header (newmsg, hdr, NULL);
   mu_mime_add_part (mime, newmsg);
   mu_message_unref (newmsg);
@@ -184,51 +180,53 @@ mime_create_ds (mu_mime_t mime, mu_message_t orig)
   mu_message_t newmsg;
   mu_stream_t stream;
   mu_header_t hdr;
-  mu_off_t off = 0;
   mu_body_t body;
   char *email;
-  char datestr[80];
-  time_t t = time (NULL);
   struct tm tm, *tmp;
-  mu_timezone tz;
+  struct mu_timezone tz;
   mu_envelope_t env;
   const char *p;
+  time_t t = time (NULL);
   
   mu_message_create (&newmsg, NULL);
   mu_message_get_header (newmsg, &hdr); 
   mu_header_set_value (hdr, "Content-Type", "message/delivery-status", 1);
   mu_message_get_body (newmsg, &body);
-  mu_body_get_stream (body, &stream);
-  mu_stream_printf (stream, &off, "Reporting-UA: sieve; %s\n", PACKAGE_STRING);
+  mu_body_get_streamref (body, &stream);
+  mu_stream_printf (stream, "Reporting-UA: sieve; %s\n", PACKAGE_STRING);
 
   mu_message_get_envelope (orig, &env);
   if (mu_envelope_sget_date (env, &p) == 0
-      && mu_parse_ctime_date_time (&p, &tm, &tz) == 0)
-    t = mu_tm2time (&tm, &tz);
+      && mu_scan_datetime (p, MU_DATETIME_FROM, &tm, &tz, NULL) == 0)
+    {
+      tmp = &tm;
+    }
   else
-    /* Use local time instead */
-    t = time (NULL);
-  tmp = localtime (&t);
+    {
+      tmp = localtime (&t);
+      mu_datetime_tz_local (&tz);
+    }
       
-  /* FIXME: timezone info is lost */
-  mu_strftime (datestr, sizeof datestr, "%a, %b %d %H:%M:%S %Y %Z", tmp);
-  mu_stream_printf (stream, &off, "Arrival-Date: %s\n", datestr);
+  mu_c_streamftime (stream, "Arrival-Date: %a, %b %d %H:%M:%S %Y %Z%n",
+		    tmp, &tz);
 
   email = mu_get_user_email (NULL);
-  mu_stream_printf (stream, &off, "Final-Recipient: RFC822; %s\n",
+  mu_stream_printf (stream, "Final-Recipient: RFC822; %s\n",
 		    email ? email : "unknown");
   free (email);
-  mu_stream_printf (stream, &off, "Action: deleted\n");
-  mu_stream_printf (stream, &off, 
+  mu_stream_printf (stream, "Action: deleted\n");
+  mu_stream_printf (stream,  
 		    "Disposition: automatic-action/MDN-sent-automatically;deleted\n");
 
-  t = time (NULL);
   tmp = localtime (&t);
-  mu_strftime (datestr, sizeof datestr, "%a, %b %d %H:%M:%S %Y %Z", tmp);
-  mu_stream_printf (stream, &off, "Last-Attempt-Date: %s\n", datestr);
+  mu_datetime_tz_local (&tz);
+  mu_c_streamftime (stream, "Last-Attempt-Date: %a, %b %d %H:%M:%S %Y %Z%n",
+		    tmp, &tz);
 
   mu_stream_close (stream);
-  mu_mime_add_part(mime, newmsg);
+  mu_stream_destroy (&stream);
+  
+  mu_mime_add_part (mime, newmsg);
   mu_message_unref (newmsg);
 }
 
@@ -240,31 +238,25 @@ mime_create_quote (mu_mime_t mime, mu_message_t msg)
   mu_message_t newmsg;
   mu_stream_t istream, ostream;
   mu_header_t hdr;
-  size_t ioff = 0, ooff = 0, n;
-  char buffer[512];
+  int rc;
   mu_body_t body;
-    
+  
   mu_message_create (&newmsg, NULL);
   mu_message_get_header (newmsg, &hdr); 
   mu_header_set_value (hdr, "Content-Type", "message/rfc822", 1);
   mu_message_get_body (newmsg, &body);
-  mu_body_get_stream (body, &ostream);
-  mu_message_get_stream (msg, &istream);
-  
-  while (mu_stream_read (istream, buffer, sizeof buffer - 1, ioff, &n) == 0
-	 && n != 0)
-    {
-      size_t sz;
-      mu_stream_write (ostream, buffer, n, ooff, &sz);
-      if (sz != n)
-	return EIO;
-      ooff += n;
-      ioff += n;
-    }
+  mu_body_get_streamref (body, &ostream);
+  mu_message_get_streamref (msg, &istream);
+
+  rc = mu_stream_copy (ostream, istream, 0, NULL);
+
+  mu_stream_destroy (&istream);
   mu_stream_close (ostream);
+  mu_stream_destroy (&ostream);
+  
   mu_mime_add_part (mime, newmsg);
   mu_message_unref (newmsg);
-  return 0;
+  return rc;
 }
   
 static int
@@ -289,7 +281,7 @@ build_mime (mu_mime_t *pmime, mu_message_t msg, const char *text)
 }
 
 static int
-sieve_action_reject (mu_sieve_machine_t mach, mu_list_t args, mu_list_t tags)
+sieve_action_reject (mu_sieve_machine_t mach)
 {
   mu_mime_t mime = NULL;
   mu_mailer_t mailer = mu_sieve_get_mailer (mach);
@@ -297,22 +289,22 @@ sieve_action_reject (mu_sieve_machine_t mach, mu_list_t args, mu_list_t tags)
   mu_message_t newmsg;
   char *addrtext;
   mu_address_t from, to;
+  mu_header_t hdr;
+  char *text;
   
-  mu_sieve_value_t *val = mu_sieve_value_get (args, 0);
-  if (!val)
-    {
-      mu_sieve_error (mach, _("reject: cannot get text!"));
-      mu_sieve_abort (mach);
-    }
+  mu_sieve_get_arg (mach, 0, SVT_STRING, &text);
   mu_sieve_log_action (mach, "REJECT", NULL);  
   if (mu_sieve_is_dry_run (mach))
     return 0;
 
-  rc = build_mime (&mime, mach->msg, val->v.string);
+  rc = build_mime (&mime, mach->msg, text);
 
   mu_mime_get_message (mime, &newmsg);
-
+  mu_message_unref (newmsg);
   mu_sieve_get_message_sender (mach->msg, &addrtext);
+  mu_message_get_header (newmsg, &hdr);
+  mu_header_prepend (hdr, MU_HEADER_TO, addrtext);
+
   rc = mu_address_create (&to, addrtext);
   if (rc)
     {
@@ -381,22 +373,26 @@ check_redirect_loop (mu_message_t msg)
   mu_header_get_field_count (hdr, &num);
   for (i = 1; !loop && i <= num; i++)
     {
-      mu_header_get_field_name (hdr, i, buf, sizeof buf, NULL);
+      if (mu_header_get_field_name (hdr, i, buf, sizeof buf, NULL))
+	continue;
       if (mu_c_strcasecmp (buf, "X-Loop-Prevention") == 0)
 	{
 	  size_t j, cnt = 0;
 	  mu_address_t addr;
 	  
-	  mu_header_get_field_value (hdr, i, buf, sizeof buf, NULL);
+	  if (mu_header_get_field_value (hdr, i, buf, sizeof buf, NULL))
+	    continue;
 	  if (mu_address_create (&addr, buf))
 	    continue;
 
-	  mu_address_get_count (addr, &cnt);
-	  for (j = 1; !loop && j <= cnt; j++)
+	  if (mu_address_get_count (addr, &cnt) == 0)
 	    {
-	      mu_address_get_email (addr, j, buf, sizeof buf, NULL);
-	      if (mu_c_strcasecmp (buf, email) == 0)
-		loop = 1;
+	      for (j = 1; !loop && j <= cnt; j++)
+		{
+		  if (mu_address_get_email (addr, j, buf, sizeof buf, NULL) == 0
+		      && mu_c_strcasecmp (buf, email) == 0)
+		    loop = 1;
+		}
 	    }
 	  mu_address_destroy (&addr);
 	}
@@ -406,7 +402,7 @@ check_redirect_loop (mu_message_t msg)
 }
 
 static int
-sieve_action_redirect (mu_sieve_machine_t mach, mu_list_t args, mu_list_t tags)
+sieve_action_redirect (mu_sieve_machine_t mach)
 {
   mu_message_t msg, newmsg = NULL;
   mu_address_t addr = NULL, from = NULL;
@@ -414,25 +410,21 @@ sieve_action_redirect (mu_sieve_machine_t mach, mu_list_t args, mu_list_t tags)
   int rc;
   char *fromaddr, *p;
   mu_mailer_t mailer = mu_sieve_get_mailer (mach);
+  char *addrstr;
   
-  mu_sieve_value_t *val = mu_sieve_value_get (args, 0);
-  if (!val)
-    {
-      mu_sieve_error (mach, _("cannot get address!"));
-      mu_sieve_abort (mach);
-    }
+  mu_sieve_get_arg (mach, 0, SVT_STRING, &addrstr);
 
-  rc = mu_address_create (&addr, val->v.string);
+  rc = mu_address_create (&addr, addrstr);
   if (rc)
     {
       mu_sieve_error (mach,
 		      _("%lu: parsing recipient address `%s' failed: %s"),
 		      (unsigned long) mu_sieve_get_message_num (mach),
-		      val->v.string, mu_strerror (rc));
+		      addrstr, mu_strerror (rc));
       return 1;
     }
   
-  mu_sieve_log_action (mach, "REDIRECT", _("to %s"), val->v.string);
+  mu_sieve_log_action (mach, "REDIRECT", _("to %s"), addrstr);
   if (mu_sieve_is_dry_run (mach))
     return 0;
 
@@ -523,31 +515,30 @@ mu_sieve_data_type fileinto_args[] = {
 };
 
 static int
-perms_tag_checker (const char *name, mu_list_t tags, mu_list_t args)
+perms_tag_checker (mu_sieve_machine_t mach)
 {
-  mu_iterator_t itr;
+  size_t i;
   int err = 0;
   
-  if (!tags || mu_list_get_iterator (tags, &itr))
+  if (mach->tagcount == 0)
     return 0;
-  for (mu_iterator_first (itr); !err && !mu_iterator_is_done (itr);
-       mu_iterator_next (itr))
+  for (i = 0; i < mach->tagcount; i++)
     {
       int flag;
       const char *p;
-      mu_sieve_runtime_tag_t *t;
-      mu_iterator_current (itr, (void **)&t);
+      mu_sieve_value_t *t = mu_sieve_get_tag_n (mach, i);
+
       if (strcmp (t->tag, "permissions") == 0)
 	{
-	  if (mu_parse_stream_perm_string (&flag, t->arg->v.string, &p))
+	  if (mu_parse_stream_perm_string (&flag, t->v.string, &p))
 	    {
-	      mu_sv_compile_error (&mu_sieve_locus, 
-				   _("invalid permissions (near %s)"), p);
+	      mu_diag_at_locus_range (MU_LOG_ERROR, &t->locus,
+				      _("invalid permissions (near %s)"), p);
+	      mu_i_sv_error (mach);
 	      err = 1;
 	    }
 	}
     }
-  mu_iterator_destroy (&itr);
   return err;
 }
 
@@ -562,7 +553,7 @@ static mu_sieve_tag_group_t fileinto_tag_groups[] = {
 };
   
 void
-mu_sv_register_standard_actions (mu_sieve_machine_t mach)
+mu_i_sv_register_standard_actions (mu_sieve_machine_t mach)
 {
   mu_sieve_register_action (mach, "stop", sieve_action_stop, NULL, NULL, 1);
   mu_sieve_register_action (mach, "keep", sieve_action_keep, NULL, NULL, 1);
