@@ -1,9 +1,10 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2001, 2002, 2003, 2005 Free Software Foundation, Inc.
+   Copyright (C) 1999, 2001-2003, 2005, 2007, 2009-2012, 2014-2017 Free
+   Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
+   the Free Software Foundation; either version 3, or (at your option)
    any later version.
 
    GNU Mailutils is distributed in the hope that it will be useful,
@@ -12,9 +13,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with GNU Mailutils; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-   MA 02110-1301 USA */
+   along with GNU Mailutils.  If not, see <http://www.gnu.org/licenses/>. */
 
 #include "mail.h"
 
@@ -33,9 +32,9 @@ mail_eq (int argc, char **argv)
     case 1:
       n = get_cursor ();
       if (n == 0)
-        util_error (_("No applicable message"));
+        mu_error (_("No applicable message"));
       else
-        fprintf (ofile, "%u\n", n);
+        mu_printf ("%lu\n", (unsigned long) n);
       break;
 
     case 2:
@@ -44,7 +43,8 @@ mail_eq (int argc, char **argv)
 	  if (list->msg_part[0] <= total)
 	    {
 	      set_cursor (list->msg_part[0]);
-	      fprintf (ofile, "%u\n", list->msg_part[0]);
+	      mu_printf ("%lu\n",
+				(unsigned long) list->msg_part[0]);
 	    }
 	  else
 	    util_error_range (list->msg_part[0]);
